
#ifndef MOCK_TABLE_HEADER
#define MOCK_TABLE_HEADER

/* DO NOT EDIT THIS FILE */
/* auto-generated by build_mock_table.pl */

#define MOCK_VA_NUM_ARGS(...) MOCK_VA_NUM_ARGS_IMPL_((__VA_ARGS__,    63, 62, 61, 60, \
                                    59, 58, 57, 56, 55, 54, 53, 52, 51, 50, \
                                    49, 48, 47, 46, 45, 44, 43, 42, 41, 40, \
                                    39, 38, 37, 36, 35, 34, 33, 32, 31, 30, \
                                    29, 28, 27, 26, 25, 24, 23, 22, 21, 20, \
                                    19, 18, 17, 16, 15, 14, 13, 12, 11, 10, \
                                     9,  8,  7,  6,  5,  4,  3,  2,  1))
#define MOCK_VA_NUM_ARGS_IMPL_(tuple) MOCK_VA_NUM_ARGS_IMPL tuple

#define MOCK_VA_NUM_ARGS_IMPL( \
                      _1, _2, _3, _4, _5, _6, _7, _8, _9,_10, \
                     _11,_12,_13,_14,_15,_16,_17,_18,_19,_20, \
                     _21,_22,_23,_24,_25,_26,_27,_28,_29,_30, \
                     _31,_32,_33,_34,_35,_36,_37,_38,_39,_40, \
                     _41,_42,_43,_44,_45,_46,_47,_48,_49,_50, \
                     _51,_52,_53,_54,_55,_56,_57,_58,_59,_60, \
                     _61,_62,_63,N,...) N


#define MOCK_macro_dispatcher(func, ...)   MOCK_macro_dispatcher_(func, MOCK_VA_NUM_ARGS(__VA_ARGS__))

// the three levels of indirection are a work-around for broken variadic macro support in Visual C++
#define MOCK_macro_dispatcher_(func, nargs)           MOCK_macro_dispatcher__(func, nargs)
#define MOCK_macro_dispatcher__(func, nargs)           MOCK_macro_dispatcher___(func, nargs)
#define MOCK_macro_dispatcher___(func, nargs)          func ## nargs

#define PP_NARG(...) MOCK_macro_dispatcher(mock, __VA_ARGS__)

//mock1 works for 1 or 0
#define mock1(test_reporter, function_name, mock_file, mock_line, arguments_string, ...)\
    mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t)__VA_ARGS__)

#define PP_RSEQ_N() \
	mock63, mock62, mock61, mock60, mock59, mock58, mock57, mock56, mock55, mock54,\
	mock53, mock52, mock51, mock50, mock49, mock48, mock47, mock46, mock45, mock44,\
	mock43, mock42, mock41, mock40, mock39, mock38, mock37, mock36, mock35, mock34,\
	mock33, mock32, mock31, mock30, mock29, mock28, mock27, mock26, mock25, mock24,\
	mock23, mock22, mock21, mock20, mock19, mock18, mock17, mock16, mock15, mock14,\
	mock13, mock12, mock11, mock10, mock9, mock8, mock7, mock6, mock5, mock4,\
	mock3, mock2, mock_, mock_

#define mock2(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1)

#define mock3(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2)

#define mock4(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3)

#define mock5(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4)

#define mock6(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5)

#define mock7(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6)

#define mock8(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7)

#define mock9(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8)

#define mock10(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9)

#define mock11(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10)

#define mock12(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11)

#define mock13(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12)

#define mock14(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13)

#define mock15(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14)

#define mock16(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15)

#define mock17(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16)

#define mock18(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17)

#define mock19(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18)

#define mock20(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19)

#define mock21(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20)

#define mock22(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21)

#define mock23(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22)

#define mock24(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23)

#define mock25(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24)

#define mock26(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25)

#define mock27(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26)

#define mock28(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27)

#define mock29(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28)

#define mock30(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29)

#define mock31(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30)

#define mock32(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31)

#define mock33(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32)

#define mock34(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33)

#define mock35(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34)

#define mock36(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35)

#define mock37(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36)

#define mock38(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37)

#define mock39(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38)

#define mock40(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39)

#define mock41(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40)

#define mock42(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41)

#define mock43(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42)

#define mock44(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43)

#define mock45(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44)

#define mock46(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45)

#define mock47(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46)

#define mock48(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47)

#define mock49(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48)

#define mock50(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49)

#define mock51(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50)

#define mock52(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51)

#define mock53(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52)

#define mock54(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53)

#define mock55(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54)

#define mock56(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54, arg55)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54, (intptr_t) arg55)

#define mock57(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54, arg55, arg56)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54, (intptr_t) arg55, (intptr_t) arg56)

#define mock58(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54, arg55, arg56, arg57)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54, (intptr_t) arg55, (intptr_t) arg56, (intptr_t) arg57)

#define mock59(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54, arg55, arg56, arg57, arg58)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54, (intptr_t) arg55, (intptr_t) arg56, (intptr_t) arg57, (intptr_t) arg58)

#define mock60(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54, arg55, arg56, arg57, arg58, arg59)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54, (intptr_t) arg55, (intptr_t) arg56, (intptr_t) arg57, (intptr_t) arg58, (intptr_t) arg59)

#define mock61(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54, arg55, arg56, arg57, arg58, arg59, arg60)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54, (intptr_t) arg55, (intptr_t) arg56, (intptr_t) arg57, (intptr_t) arg58, (intptr_t) arg59, (intptr_t) arg60)

#define mock62(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54, arg55, arg56, arg57, arg58, arg59, arg60, arg61)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54, (intptr_t) arg55, (intptr_t) arg56, (intptr_t) arg57, (intptr_t) arg58, (intptr_t) arg59, (intptr_t) arg60, (intptr_t) arg61)

#define mock63(test_reporter, function_name, mock_file, mock_line, arguments_string, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24, arg25, arg26, arg27, arg28, arg29, arg30, arg31, arg32, arg33, arg34, arg35, arg36, arg37, arg38, arg39, arg40, arg41, arg42, arg43, arg44, arg45, arg46, arg47, arg48, arg49, arg50, arg51, arg52, arg53, arg54, arg55, arg56, arg57, arg58, arg59, arg60, arg61, arg62)\
  mock_(test_reporter, function_name, mock_file, mock_line, arguments_string, (intptr_t) arg0, (intptr_t) arg1, (intptr_t) arg2, (intptr_t) arg3, (intptr_t) arg4, (intptr_t) arg5, (intptr_t) arg6, (intptr_t) arg7, (intptr_t) arg8, (intptr_t) arg9, (intptr_t) arg10, (intptr_t) arg11, (intptr_t) arg12, (intptr_t) arg13, (intptr_t) arg14, (intptr_t) arg15, (intptr_t) arg16, (intptr_t) arg17, (intptr_t) arg18, (intptr_t) arg19, (intptr_t) arg20, (intptr_t) arg21, (intptr_t) arg22, (intptr_t) arg23, (intptr_t) arg24, (intptr_t) arg25, (intptr_t) arg26, (intptr_t) arg27, (intptr_t) arg28, (intptr_t) arg29, (intptr_t) arg30, (intptr_t) arg31, (intptr_t) arg32, (intptr_t) arg33, (intptr_t) arg34, (intptr_t) arg35, (intptr_t) arg36, (intptr_t) arg37, (intptr_t) arg38, (intptr_t) arg39, (intptr_t) arg40, (intptr_t) arg41, (intptr_t) arg42, (intptr_t) arg43, (intptr_t) arg44, (intptr_t) arg45, (intptr_t) arg46, (intptr_t) arg47, (intptr_t) arg48, (intptr_t) arg49, (intptr_t) arg50, (intptr_t) arg51, (intptr_t) arg52, (intptr_t) arg53, (intptr_t) arg54, (intptr_t) arg55, (intptr_t) arg56, (intptr_t) arg57, (intptr_t) arg58, (intptr_t) arg59, (intptr_t) arg60, (intptr_t) arg61, (intptr_t) arg62)

#endif /* MOCK_TABLE_HEADER */
